@@ -375,17 +375,15 @@ RecordVal* Connection::BuildConnVal()
 		conn_val->Assign(2, resp_endp);
 		// 3 and 4 are set below.
 		conn_val->Assign(5, new TableVal(string_set));	// service
-		conn_val->Assign(6, new StringVal(""));	// addl
-		conn_val->Assign(7, new Val(0, TYPE_COUNT));	// hot
-		conn_val->Assign(8, new StringVal(""));	// history
+		conn_val->Assign(6, new StringVal(""));	// history
 
 		if ( ! uid )
 			uid.Set(bits_per_uid);
 
-		conn_val->Assign(9, new StringVal(uid.Base62("C").c_str()));
+		conn_val->Assign(7, new StringVal(uid.Base62("C").c_str()));
 
 		if ( encapsulation && encapsulation->Depth() > 0 )
-			conn_val->Assign(10, encapsulation->GetVectorVal());
+			conn_val->Assign(8, encapsulation->GetVectorVal());
 		}
 
 	if ( root_analyzer )
@@ -393,7 +391,7 @@ RecordVal* Connection::BuildConnVal()
 
 	conn_val->Assign(3, new Val(start_time, TYPE_TIME));	// ###
 	conn_val->Assign(4, new Val(last_time - start_time, TYPE_INTERVAL));
-	conn_val->Assign(8, new StringVal(history.c_str()));
+	conn_val->Assign(6, new StringVal(history.c_str()));
 
 	conn_val->SetOrigin(this);
 
