@@ -19,7 +19,7 @@ public:
 
 	void Undelivered(uint64 seq, int len, bool orig);
 
-//	static analyzer::Analyzer* InstantiateAnalyzer(Connection* conn)
+//	static analyzer::Analyzer* Instantiate(Connection* conn)
 //		{ return new File_Analyzer(conn); }
 
 protected:
@@ -38,7 +38,7 @@ public:
 		: File_Analyzer("IRC_Data", conn)
 		{ }
 
-	static Analyzer* InstantiateAnalyzer(Connection* conn)
+	static Analyzer* Instantiate(Connection* conn)
 		{ return new IRC_Data(conn); }
 };
 
@@ -48,7 +48,7 @@ public:
 		: File_Analyzer("FTP_Data", conn)
 		{ }
 
-	static Analyzer* InstantiateAnalyzer(Connection* conn)
+	static Analyzer* Instantiate(Connection* conn)
 		{ return new FTP_Data(conn); }
 };
 
