@@ -23,28 +23,12 @@ public:
 	 */
 	TaggedComponent(typename T::subtype_t subtype = 0);
 
-	/**
-	 * Copy constructor.
-	 *
-	 * @param other Another component from which to copy its tag value.
-	 */
-	TaggedComponent(const TaggedComponent& other);
-
-	/**
-	 * Assignment operator.
-	 *
-	 * @param other A component to assign.
-	 * @return The assigned object.
-	 */
-	TaggedComponent& operator=(const TaggedComponent& other);
-
 	/**
 	 * @return The component's tag.
 	 */
 	T Tag() const;
 
 private:
-
 	T tag; /**< The automatically assigned analyzer tag. */
 	static typename T::type_t type_counter; /**< Used to generate globally
 	                                             unique tags. */
@@ -56,22 +40,6 @@ TaggedComponent<T>::TaggedComponent(typename T::subtype_t subtype)
 	tag = T(++type_counter, subtype);
 	}
 
-template <class T>
-TaggedComponent<T>::TaggedComponent(const TaggedComponent<T>& other)
-	{
-	tag = other.tag;
-	}
-
-template <class T>
-TaggedComponent<T>&
-TaggedComponent<T>::operator =(const TaggedComponent<T>& other)
-	{
-	if ( &other != this )
-		tag = other.tag;
-
-	return *this;
-	}
-
 template <class T>
 T TaggedComponent<T>::Tag() const
 	{
